@@ -1,7 +1,7 @@
 // drumkv1widget_config.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,7 +30,12 @@
 #include "ui_drumkv1widget_config.h"
 
 #include <QPushButton>
+#include <QComboBox>
+
 #include <QMessageBox>
+#include <QFileDialog>
+#include <QFileInfo>
+#include <QUrl>
 
 #include <QMenu>
 
@@ -54,6 +59,12 @@ drumkv1widget_config::drumkv1widget_config (
 	//m_ui.CustomStyleThemeComboBox->addItem(tr("(default)"));
 	m_ui.CustomStyleThemeComboBox->addItems(QStyleFactory::keys());
 
+	// Note names.
+	QStringList notes;
+	for (int note = 0; note < 128; ++note)
+		notes << drumkv1_ui::noteName(note);
+	m_ui.TuningRefNoteComboBox->insertItems(0, notes);
+
 	// Setup options...
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
 	if (pConfig && m_pDrumkUi) {
@@ -85,6 +96,19 @@ drumkv1widget_config::drumkv1widget_config (
 			m_ui.ProgramsPreviewCheckBox->setEnabled(!bPlugin);
 			m_ui.ProgramsEnabledCheckBox->setChecked(pPrograms->enabled());
 		}
+		// Initialize conveniency options...
+		loadComboBoxHistory(m_ui.TuningScaleFileComboBox);
+		loadComboBoxHistory(m_ui.TuningKeyMapFileComboBox);
+		// Micro-tonal tuning settings...
+		m_ui.TuningEnabledCheckBox->setChecked(pConfig->bTuningEnabled);
+		m_ui.TuningRefNoteComboBox->setCurrentIndex(pConfig->iTuningRefNote);
+		m_ui.TuningRefPitchSpinBox->setValue(double(pConfig->fTuningRefPitch));
+		setComboBoxCurrentItem(
+			m_ui.TuningScaleFileComboBox,
+			QFileInfo(pConfig->sTuningScaleFile));
+		setComboBoxCurrentItem(
+			m_ui.TuningKeyMapFileComboBox,
+			QFileInfo(pConfig->sTuningKeyMapFile));
 	}
 
 	// Signal/slots connections...
@@ -147,6 +171,32 @@ drumkv1widget_config::drumkv1widget_config (
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(programsContextMenuRequested(const QPoint&)));
 
+	// Tuning slots...
+	QObject::connect(m_ui.TuningEnabledCheckBox,
+		SIGNAL(toggled(bool)),
+		SLOT(tuningChanged()));
+	QObject::connect(m_ui.TuningRefNoteComboBox,
+		SIGNAL(activated(int)),
+		SLOT(tuningChanged()));
+	QObject::connect(m_ui.TuningRefPitchSpinBox,
+		SIGNAL(valueChanged(double)),
+		SLOT(tuningChanged()));
+	QObject::connect(m_ui.TuningRefNotePushButton,
+		SIGNAL(clicked()),
+		SLOT(tuningRefNoteClicked()));
+	QObject::connect(m_ui.TuningScaleFileComboBox,
+		SIGNAL(activated(const QString&)),
+		SLOT(tuningChanged()));
+	QObject::connect(m_ui.TuningScaleFileToolButton,
+		SIGNAL(clicked()),
+		SLOT(tuningScaleFileClicked()));
+	QObject::connect(m_ui.TuningKeyMapFileToolButton,
+		SIGNAL(clicked()),
+		SLOT(tuningKeyMapFileClicked()));
+	QObject::connect(m_ui.TuningKeyMapFileComboBox,
+		SIGNAL(activated(const QString&)),
+		SLOT(tuningChanged()));
+
 	// Options slots...
 	QObject::connect(m_ui.ProgramsPreviewCheckBox,
 		SIGNAL(toggled(bool)),
@@ -176,6 +226,7 @@ drumkv1widget_config::drumkv1widget_config (
 		SLOT(reject()));
 
 	// Dialog dirty flags.
+	m_iDirtyTuning   = 0;
 	m_iDirtyControls = 0;
 	m_iDirtyPrograms = 0;
 	m_iDirtyOptions  = 0;
@@ -396,6 +447,114 @@ void drumkv1widget_config::programsActivated (void)
 }
 
 
+// tuning command slots
+void drumkv1widget_config::tuningRefNoteClicked (void)
+{
+	m_ui.TuningRefNoteComboBox->setCurrentIndex(69);
+	m_ui.TuningRefPitchSpinBox->setValue(double(440.0f));
+
+	tuningChanged();
+}
+
+
+void drumkv1widget_config::tuningScaleFileClicked (void)
+{
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig == NULL)
+		return;
+
+	QString sTuningScaleFile = m_ui.TuningScaleFileComboBox->currentText();
+
+	const QString  sExt("scl");
+	const QString& sTitle  = tr("Open Scale File") + " - " DRUMKV1_TITLE;
+	const QString& sFilter = tr("Scale files (*.%1)").arg(sExt);
+
+	QWidget *pParentWidget = NULL;
+	QFileDialog::Options options = 0;
+	if (pConfig->bDontUseNativeDialogs) {
+		options |= QFileDialog::DontUseNativeDialog;
+		pParentWidget = QWidget::window();
+	}
+#if 1//QT_VERSION < 0x040400
+	sTuningScaleFile = QFileDialog::getOpenFileName(pParentWidget,
+		sTitle, pConfig->sTuningScaleDir, sFilter, NULL, options);
+#else
+	QFileDialog fileDialog(pParentWidget,
+		sTitle, sTuningScaleFile, sFilter);
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFiles);
+	fileDialog.setDefaultSuffix(sExt);
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(pConfig->sTuningScaleDir));
+	fileDialog.setSidebarUrls(urls);
+	fileDialog.setOptions(options);
+	if (fileDialog.exec())
+		sTuningScaleFile = fileDialog.selectedFiles().first();
+#endif
+
+	if (!sTuningScaleFile.isEmpty()) {
+		const QFileInfo info(sTuningScaleFile);
+		if (setComboBoxCurrentItem(m_ui.TuningScaleFileComboBox, info)) {
+			pConfig->sTuningScaleDir = info.absolutePath();
+			tuningChanged();
+		}
+	}
+}
+
+
+void drumkv1widget_config::tuningKeyMapFileClicked (void)
+{
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig == NULL)
+		return;
+
+	QString sTuningKeyMapFile = m_ui.TuningKeyMapFileComboBox->currentText();
+
+	const QString  sExt("kbm");
+	const QString& sTitle  = tr("Open Key Map File") + " - " DRUMKV1_TITLE;
+	const QString& sFilter = tr("Key Map files (*.%1)").arg(sExt);
+
+	QWidget *pParentWidget = NULL;
+	QFileDialog::Options options = 0;
+	if (pConfig->bDontUseNativeDialogs) {
+		options |= QFileDialog::DontUseNativeDialog;
+		pParentWidget = QWidget::window();
+	}
+#if 1//QT_VERSION < 0x040400
+	sTuningKeyMapFile = QFileDialog::getOpenFileName(pParentWidget,
+		sTitle, pConfig->sTuningKeyMapDir, sFilter, NULL, options);
+#else
+	QFileDialog fileDialog(pParentWidget,
+		sTitle, sTuningScaleFile, sFilter);
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFiles);
+	fileDialog.setDefaultSuffix(sExt);
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(pConfig->sTuningKeyMapDir));
+	fileDialog.setSidebarUrls(urls);
+	fileDialog.setOptions(options);
+	if (fileDialog.exec())
+		sTuningKeyMapFile = fileDialog.selectedFiles().first();
+#endif
+
+	if (!sTuningKeyMapFile.isEmpty()) {
+		const QFileInfo info(sTuningKeyMapFile);
+		if (setComboBoxCurrentItem(m_ui.TuningKeyMapFileComboBox, info)) {
+			pConfig->sTuningKeyMapDir = info.absolutePath();
+			tuningChanged();
+		}
+	}
+}
+
+
+void drumkv1widget_config::tuningChanged (void)
+{
+	++m_iDirtyTuning;
+
+	stabilize();
+}
+
+
 // options slot.
 void drumkv1widget_config::optionsChanged (void)
 {
@@ -425,8 +584,23 @@ void drumkv1widget_config::stabilize (void)
 	m_ui.ProgramsEditToolButton->setEnabled(bEnabled);
 	m_ui.ProgramsDeleteToolButton->setEnabled(bEnabled);
 
+	bEnabled = m_ui.TuningEnabledCheckBox->isChecked();
+	m_ui.TuningRefNoteTextLabel->setEnabled(bEnabled);
+	m_ui.TuningRefNoteComboBox->setEnabled(bEnabled);
+	m_ui.TuningRefPitchSpinBox->setEnabled(bEnabled);
+	m_ui.TuningRefNotePushButton->setEnabled(bEnabled);
+	m_ui.TuningScaleFileTextLabel->setEnabled(bEnabled);
+	m_ui.TuningScaleFileComboBox->setEnabled(bEnabled);
+	m_ui.TuningScaleFileToolButton->setEnabled(bEnabled);
+	m_ui.TuningKeyMapFileTextLabel->setEnabled(bEnabled);
+	m_ui.TuningKeyMapFileComboBox->setEnabled(bEnabled);
+	m_ui.TuningKeyMapFileToolButton->setEnabled(bEnabled);
+
 	const bool bValid
-		= (m_iDirtyControls > 0 || m_iDirtyPrograms > 0 || m_iDirtyOptions > 0);
+		= (m_iDirtyTuning   > 0
+		|| m_iDirtyControls > 0
+		|| m_iDirtyPrograms > 0
+		|| m_iDirtyOptions  > 0);
 	m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->setEnabled(bValid);
 }
 
@@ -436,6 +610,22 @@ void drumkv1widget_config::accept (void)
 {
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
 
+	if (m_iDirtyTuning > 0 && pConfig && m_pDrumkUi) {
+		// Micro-tonal tuning settings...
+		pConfig->bTuningEnabled = m_ui.TuningEnabledCheckBox->isChecked();
+		pConfig->iTuningRefNote = m_ui.TuningRefNoteComboBox->currentIndex();
+		pConfig->fTuningRefPitch = float(m_ui.TuningRefPitchSpinBox->value());
+		pConfig->sTuningScaleFile = comboBoxCurrentItem(m_ui.TuningScaleFileComboBox);
+		pConfig->sTuningKeyMapFile = comboBoxCurrentItem(m_ui.TuningKeyMapFileComboBox);
+		// Reset/update micro-tonal tuning...
+		m_pDrumkUi->updateTuning();
+		// Save other conveniency options...
+		saveComboBoxHistory(m_ui.TuningScaleFileComboBox);
+		saveComboBoxHistory(m_ui.TuningKeyMapFileComboBox);
+		// Reset dirty flag.
+		m_iDirtyTuning = 0;
+	}
+
 	if (m_iDirtyControls > 0 && pConfig && m_pDrumkUi) {
 		// Save controls...
 		drumkv1_controls *pControls = m_pDrumkUi->controls();
@@ -508,7 +698,10 @@ void drumkv1widget_config::reject (void)
 	bool bReject = true;
 
 	// Check if there's any pending changes...
-	if (m_iDirtyControls > 0 || m_iDirtyPrograms > 0 || m_iDirtyOptions > 0) {
+	if (m_iDirtyTuning   > 0 ||
+		m_iDirtyControls > 0 ||
+		m_iDirtyPrograms > 0 ||
+		m_iDirtyOptions  > 0) {
 		QMessageBox::StandardButtons buttons
 			= QMessageBox::Discard | QMessageBox::Cancel;
 		if (m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->isEnabled())
@@ -533,4 +726,88 @@ void drumkv1widget_config::reject (void)
 }
 
 
+// Combo box history persistence helper implementation.
+void drumkv1widget_config::loadComboBoxHistory ( QComboBox *pComboBox )
+{
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig == NULL)
+		return;
+
+	// Load combobox list from configuration settings file...
+	const bool bBlockSignals = pComboBox->blockSignals(true);
+	pConfig->beginGroup("/History");
+	const QStringList& history
+		= pConfig->value('/' + pComboBox->objectName()).toStringList();
+	QStringListIterator iter(history);
+	while (iter.hasNext()) {
+		const QFileInfo info(iter.next());
+		if (info.exists() && info.isReadable()) {
+			const QString& sPath = info.canonicalFilePath();
+			pComboBox->insertItem(0, info.fileName(), sPath);
+		}
+	}
+	pConfig->endGroup();
+	pComboBox->blockSignals(bBlockSignals);
+}
+
+
+void drumkv1widget_config::saveComboBoxHistory ( QComboBox *pComboBox )
+{
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig == NULL)
+		return;
+
+	// Save combobox list to configuration settings file...
+	const bool bBlockSignals = pComboBox->blockSignals(true);
+	pConfig->beginGroup("/History");
+	QStringList history;
+	const int iCount = pComboBox->count();
+	for (int i = 0; i < iCount; ++i) {
+		const QString& sData = pComboBox->itemData(i).toString();
+		if (!sData.isEmpty())
+			history.prepend(sData);
+	}
+	pConfig->setValue('/' + pComboBox->objectName(), history);
+	pConfig->endGroup();
+	pComboBox->blockSignals(bBlockSignals);
+}
+
+
+// Combo box settter/gettter helper prototypes.
+bool drumkv1widget_config::setComboBoxCurrentItem (
+	QComboBox *pComboBox, const QFileInfo& info )
+{
+	const bool bBlockSignals = pComboBox->blockSignals(true);
+	const bool bResult = info.exists() && info.isReadable();
+	if (bResult) {
+		const QString& sData = info.canonicalFilePath();
+		int iIndex = pComboBox->findData(sData);
+		if (iIndex < 0) {
+			pComboBox->insertItem(0, info.fileName(), sData);
+			iIndex = 0;
+		}
+		pComboBox->setCurrentIndex(iIndex);
+		pComboBox->setToolTip(sData);
+	} else {
+		pComboBox->setCurrentIndex(pComboBox->count() - 1);
+		pComboBox->setToolTip(pComboBox->currentText());
+	}
+	pComboBox->blockSignals(bBlockSignals);
+
+	return bResult;
+}
+
+
+QString drumkv1widget_config::comboBoxCurrentItem ( QComboBox *pComboBox )
+{
+	QString sData;
+
+	const int iIndex = pComboBox->currentIndex();
+	if (iIndex >= 0)
+		sData = pComboBox->itemData(iIndex).toString();
+
+	return sData;
+}
+
+
 // end of drumkv1widget_config.cpp
